@@ -31,7 +31,7 @@
 
 
 /*
- * $Id: dlsof.h,v 1.16 2006/03/27 22:57:11 abe Exp $
+ * $Id: dlsof.h,v 1.18 2008/10/21 16:17:21 abe Exp $
  */
 
 
@@ -40,6 +40,7 @@
 
 #include <dirent.h>
 #define	DIRTYPE	dirent			/* for arg.c's enter_dir() */
+#define	__USE_GNU			/* to get all O_* symbols in fcntl.h */
 #include <fcntl.h>
 #include <malloc.h>
 #include <signal.h>
@@ -57,9 +58,15 @@
 
 #include <rpc/rpc.h>
 #include <rpc/pmap_prot.h>
+ 
+#if	defined(HASSELINUX)
+#include <selinux/selinux.h>
+#endif	/* defined(HASSELINUX) */
+
 #include <sys/sysmacros.h>
 #include <sys/socket.h>
 #include <arpa/inet.h>
+#include <linux/if_ether.h>
 
 
 /*
@@ -159,6 +166,7 @@ struct sfile {
 	struct sfile *next;		/* forward link */
 };
 
-extern	int	HasNFS;
+extern int HasNFS;
+extern int OffType;
 
 #endif	/* LINUX_LSOF_H	*/
