@@ -32,7 +32,7 @@
 #ifndef lint
 static char copyright[] =
 "@(#) Copyright 1994 Purdue Research Foundation.\nAll rights reserved.\n";
-static char *rcsid = "$Id: dproc.c,v 1.32 2005/08/08 19:55:41 abe Exp $";
+static char *rcsid = "$Id: dproc.c,v 1.34 2008/10/21 16:16:42 abe Exp $";
 #endif
 
 #include "lsof.h"
@@ -191,6 +191,16 @@ close_kvm()
 void
 gather_proc_info()
 {
+	short cckreg;			/* conditional status of regular file
+					 * checking:
+					 *     0 = unconditionally check
+					 *     1 = conditionally check */
+	short ckscko;			/* socket file only checking status:
+					 *     0 = none
+					 *     1 = check only socket files,
+					 *	   including TCP and UDP
+					 *	   streams with eXPORT data,
+					 *	   where supported */
 	static int ft = 1;
 	int i, j;
 	struct proc *p;
@@ -225,10 +235,13 @@ gather_proc_info()
 	znhash_t *zp, *zpn;
 #endif	/* defined(HASZONES) */
 
+	if (ft) {
 /*
  * Do first-time only operations.
  */
-	if (ft) {
+	/*
+	 * Get the segment vnodeops address.
+	 */
 	    if (get_Nl_value("sgvops", Drive_Nl, &Sgvops) < 0)
 		Sgvops = (KA_T)NULL;
 	    ft = 0;
@@ -252,6 +265,55 @@ gather_proc_info()
 #endif	/* solaris>=20501 */
 
 	}
+/*
+ * Define socket and regular file conditional processing flags.
+ *
+ * If only socket files have been selected, or socket files have been
+ * selected, ANDed with other selection options, enable the skipping of
+ * regular files.
+ *
+ * If socket files and some process options have been selected, enable
+ * conditional skipping of regular file; i.e., regular files will be skipped
+ * unless they belong to a process selected by one of the specified options.
+ */
+	if (Selflags & SELNW) {
+
+	/*
+	 * Some network files selection options have been specified.
+	 */
+	    if (Fand || !(Selflags & ~SELNW)) {
+
+	    /*
+	     * Selection ANDing or only network file options have been
+	     * specified, so set unconditional skipping of regular files
+	     * and socket file only checking.
+	     */
+		cckreg = 0;
+		ckscko = 1;
+	    } else {
+
+	    /*
+	     * If ORed file selection options have been specified, or no
+	     * ORed process selection options have been specified, enable
+	     * unconditional file checking and clear socket file only
+	     * checking.
+	     *
+	     * If only ORed process selection options have been specified,
+	     * enable conditional file skipping and socket file only checking.
+	     */
+		if ((Selflags & SELFILE) || !(Selflags & SELPROC))
+		    cckreg = ckscko = 0;
+		else
+		    cckreg = ckscko = 1;
+	    }
+	} else {
+
+	/*
+	 * No network file selection options were specified.  Enable
+	 * unconditional file checking and clear socket file only checking.
+	 */
+	    cckreg = ckscko = 0;
+	}
 /*
  * Read the process table.
  */
@@ -313,6 +375,15 @@ gather_proc_info()
 	 */
 	    if (is_cmd_excl(u->u_comm, &pss, &sf))
 		continue;
+	    if (cckreg) {
+
+	    /*
+	     * If conditional checking of regular files is enabled, enable
+	     * socket file only checking, based on the process' selection
+	     * status.
+	     */
+		ckscko = (sf & SELPROC) ? 0 : 1;
+	    }
 	    alloc_lproc(pid, pgid, (int)p->p_ppid, (UID_ARG)uid, u->u_comm,
 		(int)pss, (int)sf);
 	    Plf = (struct lfile *)NULL;
@@ -325,7 +396,7 @@ gather_proc_info()
 	 *	o Skip processes excluded by zone name.
 	 *	o Save zone name.
 	 */
-	    if (Fzone && zn[0]) {
+	    if (!ckscko && Fzone && zn[0]) {
 		zh = hash_zn(zn);
 		if (ZoneArg) {
 
@@ -390,7 +461,7 @@ gather_proc_info()
 	/*
 	 * Save current working directory information.
 	 */
-	    if (u->u_cdir) {
+	    if (!ckscko && u->u_cdir) {
 		alloc_lfile(CWD, -1);
 
 #if	defined(FILEPTR)
@@ -404,7 +475,7 @@ gather_proc_info()
 	/*
 	 * Save root directory information.
 	 */
-	    if (u->u_rdir) {
+	    if (!ckscko && u->u_rdir) {
 		alloc_lfile(RTD, -1);
 
 #if	defined(FILEPTR)
@@ -418,7 +489,7 @@ gather_proc_info()
 	/*
 	 * Save information on text files.
 	 */
-	    if (p->p_as && Sgvops) {
+	    if (!ckscko && p->p_as && Sgvops) {
 
 #if	defined(FILEPTR)
 		FILEPTR = (struct file *)NULL;
@@ -615,6 +686,26 @@ get_kernel_access()
 	if (statsafely(ALLKMEM, &sb) == 0)
 	    HasALLKMEM = 1;
 
+#if	defined(HASVXFSUTIL)
+/*
+ * If the VXFS utility library is being used, attempt to get the VXFS inode
+ * offsets before setgid permission is surrendered.
+ */
+	if (access_vxfs_ioffsets() && !Fwarn) {
+
+	/*
+	 * Warn that the VxFS offsets are unavailable.
+	 */
+	    (void) fprintf(stderr,
+		"%s: WARNING: vxfsu_get_ioffsets() returned an error.\n", Pn);
+	    (void) fprintf(stderr,
+		"%s: WARNING: Thus, no vx_inode information is available\n",
+		Pn);
+	    (void) fprintf(stderr,
+		"%s: WARNING: for display or selection of VxFS files.\n", Pn);
+	}
+#endif	/* defined(HASVXFSUTIL) */
+
 #if	defined(WILLDROPGID)
 /*
  * If Solaris kernel memory is coming from KMEM, the process is willing to
